/*************************************************************************/
/* File:        camera.v
 * Author:      Michael Phan-Ba
 *
 * Description: Interface to the CCD camera
 *              All this does is outputEnable the incoming pixel data and generate
 *              the pixel address for each data.  Asserts data valid when
 *              pixels are valid
 * Revsions:
 *   1.0        Initial design - 11/30/2009 - Michael Phan-Ba
 **************************************************************************/
`timescale 1ns/1ns

module camera(
  input             clk,   // clock
  input             reset,   // reset
  input  [11:0]     CCD_DATA,  // Input from camera
  input             CCD_FVAL,  // In Frame signal
  input             CCD_LVAL,  // In Line signal
  input             iSTART, // Start button
  input             iEND,   // Stop button
  output [11:0]     pixelOut,  // Output pixel data
  output reg [15:0] colAddr, // Pixel address
  output reg [15:0] lineAddr, // Line address
  output            newFrame,     // Done with this frame, ready to move on
  output            pixelValid);  // Pixel valid

  // Latches
  reg fvalPrev;
  reg lvalPrev;
  reg [11:0] CCD_DATA_L0;
  reg [11:0] CCD_DATA_L1;

  // Control signals
  reg outputEnableSignal;
  reg outputEnable;
  wire isNewPixel;
  wire isNewFrame;
  wire isNewLine;

  // Control signals
  assign isNewPixel = CCD_FVAL && CCD_LVAL;
  assign isNewFrame = fvalPrev && !CCD_FVAL;
  assign isNewLine = lvalPrev && !CCD_LVAL;

  assign pixelOut = CCD_DATA_L1;
  assign pixelValid = isNewPixel && outputEnable;
  assign newFrame = isNewFrame && outputEnable;

  // Latch signals
  always @(posedge clk) begin
    fvalPrev <= CCD_FVAL;
    lvalPrev <= CCD_LVAL;
    CCD_DATA_L0 <= CCD_DATA;
    CCD_DATA_L1 <= CCD_DATA_L0;
  end

  // Calculate addresses
  always @(posedge clk) begin
    if (reset || isNewLine) colAddr <= 0;
    else if (isNewPixel) colAddr <= colAddr + 16'd1;
    if (reset || isNewFrame) lineAddr <= 0;
    else if (isNewLine) lineAddr <= lineAddr + 16'd1;
  end

  // Implement START/STOP picture
  always @(posedge clk) begin
    if (reset || iSTART) outputEnableSignal <= 1;
    else if (iEND) outputEnableSignal <= 0;
    if (isNewFrame) outputEnable <= outputEnableSignal;
  end

endmodule // camera

