/*************************************************************************/
/* File:        display.v
 * Author:      Carl Ebeling
 *
 * Description: Interface to the LCD Display
 *              Receives data from the frame buffer and sends to LCD
 * Revsions:
 *   1.0        Initial design - 10/17/2009 - Carl Ebeling
 *   1.1        Cleaned up - 12/10/2009 - Carl Ebeling
 **************************************************************************/
`timescale 1ns/1ns

module  display
  (//   Host Side
   input [23:0] RGB,
   //   LCM Side
   output [7:0] LCM_DATA,
   output reg   LCM_VSYNC,
   output reg   LCM_HSYNC,
   output       LCM_DCLK,
   output       LCM_GRST,
   output       LCM_SHDB,
   //   Control Signals
   output reg   takingPixel,  // Assert when done with current pixel
   output 		newFrame,
   input        CLK_18,
   input        reset,
   input        test  // Optional test input
   );
   //   Internal Register and Wire
   reg [10:0]   hcount;
   reg [10:0]   vcount;
   reg [1:0]    RGB_state;          // Mode for RGB?

   reg [7:0]    testData;
   wire [1:0]   mSEL;

   parameter SIMULATE=0;
   // Sequence is RED, GREEN, BLUE
   localparam RED = 2'b00, GREEN = 2'b01, BLUE = 2'b10;

   //   Horizontal Parameter    ( Pixel )
   parameter    H_SYNC_CYC  =   1;
   parameter    H_SYNC_BACK =   SIMULATE==1?  3:   151;  // Back porch
   parameter    H_SYNC_ACT  =   SIMULATE==1? 30:   960;  // /3 pixels
   parameter    H_SYNC_FRONT=   SIMULATE==1?  5:   59;   // Front porch
   parameter    H_SYNC_TOTAL=   H_SYNC_CYC+H_SYNC_BACK+H_SYNC_ACT+H_SYNC_FRONT; // 1171;
   //   Virtical Parameter      ( Line )
   parameter    V_SYNC_CYC  =   1;
   parameter    V_SYNC_BACK =   SIMULATE==1?  2:     13;   // Back porch
   parameter    V_SYNC_ACT  =   SIMULATE==1? 10:    240;   // lines
   parameter    V_SYNC_FRONT=   SIMULATE==1?  1:      8;    // Front porch
   parameter    V_SYNC_TOTAL=   V_SYNC_CYC+V_SYNC_BACK+V_SYNC_ACT+V_SYNC_FRONT; // 262;

   wire [7:0] red = RGB[23:16];
   wire [7:0] green = RGB[15:8];
   wire [7:0] blue = RGB[7:0];

   // Select which color to display - stripe the display for test pattern
   assign  mSEL =   (vcount<94)                 ?   2'b01   :
                    (vcount>=94 && vcount<174)  ?   2'b10   :
                    2'b00  ;

   assign LCM_SHDB  =   1'b1;
   assign LCM_GRST  =   1'b1;
   assign LCM_DCLK  =   ~CLK_18;   // Transfer on negative edge of clock
   assign LCM_DATA  = test ?
					  ( (RGB_state == mSEL) ?   testData    :   8'h00 )  :
                      (RGB_state == RED)    ?   red    :
                      (RGB_state == GREEN)  ?   green  :
                      blue ;

   assign   newFrame = hcount==0 && vcount==0;

   always@(posedge CLK_18) begin
      if(reset) begin
         RGB_state      <=  RED;
         takingPixel  <=  1'b0;
         testData   <=  8'h00;
      end else begin
         if (hcount > H_SYNC_BACK-2 && hcount < (H_SYNC_TOTAL-H_SYNC_FRONT-2) &&
             vcount > V_SYNC_BACK+1 && vcount < (V_SYNC_TOTAL-V_SYNC_FRONT+1)) begin
// We are in the active image (with some extra pixels?)
// RGB_state counts R, G, B, R, G, B
            if (RGB_state < BLUE) begin
               RGB_state <= RGB_state + 1;
            end else begin
               RGB_state <= RED;
               if (testData < 255) testData <= testData+1;
            end
// Assert request when RGB_state is GREEN
            if(RGB_state == GREEN) begin
               takingPixel <= 1;
            end else begin
               takingPixel <= 0;
			end
// We are outside the active image
         end else begin
            RGB_state <=  RED;
            takingPixel <= 0;
            testData <=    0;
         end
      end
   end

   //   H_Sync Generator, Ref. 18.42 MHz Clock
   always@(posedge CLK_18) begin
      if (reset) begin
         hcount <=  0;
         LCM_HSYNC <=  0;
      end else begin
         // H_Sync Counter
         if (hcount < H_SYNC_TOTAL) hcount <= hcount+1;
         else hcount <=  0;
         // H_Sync Generator
		 LCM_HSYNC <= hcount >= H_SYNC_CYC;
      end
   end

   //   V_Sync Generator, Ref. H_Sync
   always@(posedge CLK_18) begin
      if(reset) begin
         vcount <=  0;
         LCM_VSYNC <=  0;
      end else begin
         // When H_Sync Re-start
         if (hcount == 0) begin
            if (vcount < V_SYNC_TOTAL) vcount <= vcount+1;
            else vcount <= 0;
			LCM_VSYNC <= vcount >= V_SYNC_CYC;
         end
      end
   end

endmodule // display
