/*************************************************************************/
/* File:        displayControl.v
 * Author:      Carl Ebeling
*
 * Description: Generate the read addresses for the display module and 
 *              deal with the handshakes with the frame buffer
 * 
 * Revsions:
 *   1.0        Initial design - 10/20/2009 - Carl Ebeling
 **************************************************************************/
`timescale 1ns/1ns

module displayControl
  (input clk,
   input reset,
   output reg [7:0] lineAddr,  // Line and column addresses
   output reg [8:0] colAddr,
   output read,            // Memory read
   input [15:0] rdData,    // Data from memory
   output reg [7:0] red,       // Data to display - with pipeline register
   output reg [7:0] green,
   output reg [7:0] blue,
   output dataValid,
   input takingPixel,
   input newFrame
   );
   
   parameter SIMULATE=0;
   localparam 
	 LINESIZE = (SIMULATE==1? 10 :320), // Number of pixels
	 LINENUM = (SIMULATE==1?   5: 240);

   reg 	 pipeFilled;     // Asserted when our read data register has been filled
   
   assign read = !pipeFilled || takingPixel;
   assign dataValid = takingPixel;   // Read guaranteed to take place: read register has been filled
   always @(posedge clk) begin
	  if (reset) begin
		 lineAddr <= 0;
		 colAddr <= 0;
		 pipeFilled <= 0;
	  end else begin
		 if (newFrame) begin
			lineAddr <= 0;
			colAddr <= 0;
			pipeFilled <= 0;
		 end else if (read) begin
            { red, green, blue } <= { {rdData[15:11], 3'b0}, {rdData[10:5], 2'b0}, {rdData[4:0], 3'b0}};
			pipeFilled <= 1;
			// Increment pixel and line counters
			if (colAddr < (LINESIZE-1)) begin
			   colAddr <= colAddr + 1;
			end else begin
			   colAddr <= 0;
			   if (lineAddr < (LINENUM-1)) begin
				  lineAddr <= lineAddr + 1;
			   end else begin
				  lineAddr <= 0;   // newFrame should do this
			   end
			end
		 end // if (!pipeFilled || takingPixel)
	  end // else: !if(reset)
   end // always @ (posedge clk)

endmodule // displayControl
