/*************************************************************************/
/* File:        frameBuffer.v
 * Author:      Carl Ebeling
 *
 * Description: Frame Buffer
 *              Includes frame buffer controller, display controller and memory
 *              Keeps track of the frame pointers
 *              Each frame is 240x320 and the pixel address is
 *              [ col(9), line(8) ] this is 17 bits.  The frame offset in
 *              the buffer is added to col, which become 10 bits (0-960)
 *              There are three frames at addresses 0, 320, 640
 * Revsions:
 *   1.0        Initial design - 10/21/2009 - Carl Ebeling
 **************************************************************************/
`timescale 1ns/1ns

module frameBuffer
  (input clk,
   input clk2,   // 2x frequency clock
   input reset,
   // Write (input) port 
   input [7:0] lineAddrIn,
   input [8:0] colAddrIn,
   input [15:0] RGBIn,
   input validPixelIn,
   output takingPixelIn,
   input newFrameIn,
   output frameReadyIn,
   // Read (output) port
   output [7:0] red,       // Data to display
   output [7:0] green,
   output [7:0] blue,
   output validPixelOut,
   input takingPixelOut,
   input newFrameOut,
   output frameReadyOut,
   // SRAM ports
   inout  [15:0] SRAM_DQ,	//	SRAM Data bus 16 Bits
   output [17:0] SRAM_ADDR, //	SRAM Address bus 18 Bits
   output SRAM_WE_N			//	SRAM Write Enable
);
   
   parameter SIMULATE=0;

   wire [7:0] lineAddrOut;
   wire [8:0] colAddrOut;
   wire [15:0] rdData;
   wire 	   read;
   
   //**********************************************************************
   // Display Contoller
   //**********************************************************************
   displayControl #(.SIMULATE(SIMULATE)) dpc
	 (.clk								(clk),
	  .reset							(reset),
	  // SRAM interface
	  .lineAddr							(lineAddrOut[7:0]),
	  .colAddr  						(colAddrOut[8:0]),
	  .read								(read),
	  .rdData							(rdData[15:0]),
	  // Display interface
	  .red                              (red[7:0]),
	  .green                            (green[7:0]),
	  .blue                             (blue[7:0]),
	  .dataValid                        (validPixelOut),
	  .takingPixel						(takingPixelOut),
	  .newFrame							(newFrameOut));

   //**********************************************************************
   // Frome buffer control - rotate the read and write pointers
   // as frames move through the buffer
   //**********************************************************************
   localparam FRAMESIZE = 320;
   reg [9:0] inPtr, outPtr;   // Frame pointers
   reg [9:0] nxtInPtr, nxtOutPtr;   // Precomputed next frame pointers
   
   always @(*) begin
	  if (inPtr == 2*FRAMESIZE) nxtInPtr = 0;
	  else nxtInPtr = inPtr + FRAMESIZE;
	  if (outPtr == 2*FRAMESIZE) nxtOutPtr = 0;
	  else nxtOutPtr = outPtr + FRAMESIZE;
   end

   // The next frame is ready if the other pointer isn't using it
   assign frameReadyIn = nxtInPtr != outPtr;
   assign frameReadyOut = nxtOutPtr != inPtr;

   always @(posedge clk) begin
	  if (reset) begin
		 outPtr = 0;
		 inPtr = FRAMESIZE;
	  end else if (newFrameIn && frameReadyIn) begin
		 inPtr <= nxtInPtr;
	  end else if (newFrameOut && frameReadyOut) begin
		 outPtr <= nxtOutPtr;
	  end
   end // always @ (posedge clk)

   wire [9:0]  colOffsetIn = colAddrIn + inPtr;
   wire [17:0] wrAddr = { colOffsetIn[9:0], lineAddrIn[7:0] };

   wire [9:0]  colOffsetOut = colAddrOut + outPtr;
   wire [17:0] rdAddr = { colOffsetOut[9:0], lineAddrOut[7:0] };
   
   //**********************************************************************
   // Frame Buffer memory - 256Kx16
   // Reads are always performed, writes when no read being done
   //**********************************************************************
   sramDP memory
	 (
	  // Outputs
	  .takingWrite			(takingPixelIn),   // RAM performing the write
	  .rdData				(rdData[15:0]),
	  .SRAM_ADDR			(SRAM_ADDR[17:0]),
	  .SRAM_WE_N			(SRAM_WE_N),
	  // Inouts
	  .SRAM_DQ				(SRAM_DQ[15:0]),
	  // Inputs
	  .reset				(reset),
	  .clk					(clk),
	  .clk2					(clk2),
	  .write				(validPixelIn),
	  .wrAddr				(wrAddr[17:0]),
	  .wrData				(RGBIn[15:0]),
	  .read					(read),
	  .rdAddr				(rdAddr[17:0]));

endmodule 
