/*************************************************************************/
/* File:        inputAsyncInf.v
 * Author:      Carl Ebeling
 *
 * Description: inputAsyncInf
 *              Provides a wrapper for the pattern generator -> frame buffer async fifo
 * Revsions:
 *   1.0        Initial design - 10/21/2009 - Carl Ebeling
 *   2.0        Separated from frame buffer - 11/10/2009 - Brian Van Essen
 **************************************************************************/
`timescale 1ns/1ns

module inputAsyncInf
  (input fbClk,
   input inClk,    // Input data stream clock (camera)
   input fbReset,
   input inReset,
   // Write (input) ports - Pattern Generator Side
   input [7:0] pgLineAddrIn,
   input [8:0] pgColAddrIn,
   input [15:0] pgRGB,
   input pgValidPixelIn,
   output pgTakingPixelIn,
   input pgNewFrameIn,
   output reg pgNewFrameInACK,
   output reg pgFrameReadyIn,
   // Read (output) ports - Frame Buffer Side
   output [7:0] fbLineAddrIn,
   output [8:0] fbColAddrIn,
   output [15:0] fbRGBIn,
   output 	   fbDataValidIn, 
   input           fbTakingDataIn,
   input 	   fbFrameReadyIn,
   output 	   fbNewFrameInInt
);
   
   parameter SIMULATE=0;

   reg        fbNewFrameIn;

   // Synchronizer
   reg 		   pgNewFrameInHold, pgNewFrameIn1, fbNewFrameInPrev;
   always @(posedge fbClk) begin
	  pgNewFrameIn1 <= pgNewFrameInHold;
	  fbNewFrameIn <= pgNewFrameIn1;
	  fbNewFrameInPrev <= fbNewFrameIn;
   end
   // Turn newFrameIn into a one-clock signal
   assign fbNewFrameInInt = !fbNewFrameInPrev && fbNewFrameIn;
   
   // Synchronizer: move newFrameInACK from frame buffer to input
   reg   pgNewFrameInACK1, fbNewFrameInACK;
   always @(posedge inClk) begin
	  pgNewFrameInACK1 <= fbNewFrameInACK;
	  pgNewFrameInACK <= pgNewFrameInACK1;
   end

   // We need to use a 4-phase handshake for pgnewFrameIn: we hold pgNewFrameInHold
   // high until we get the pgNewFrameInACK from the frame buffer
   always @(posedge inClk) begin
	  if (inReset) begin
		 pgNewFrameInHold <= 0;
	  end else begin
		 if (pgNewFrameIn) begin
			pgNewFrameInHold <= 1;
		 end
		 if (pgNewFrameInACK) begin
			pgNewFrameInHold <= 0;
		 end
	  end
   end // always @ (posedge inClk)
	  
   // Simple sequential logic in the Frame Buffer that controls newFrameInACK
   // using the 4-phase handshake
   always @(posedge fbClk) begin
	  if (fbReset) begin
		 fbNewFrameInACK <= 0;
	  end else begin
		 fbNewFrameInACK <= fbNewFrameIn;
	  end
   end

   // We only need to pass along the frameReadyIn signal since we know that
   // it will be asserted until newFrame is asserted
   // Synchronizer: move frameReadyIn from the frame buffer to the input
   reg   pgFrameReadyIn1;
   always @(posedge inClk) begin
	  pgFrameReadyIn1 <= fbFrameReadyIn;
	  pgFrameReadyIn <= pgFrameReadyIn1;
   end

   wire [7:0] fbRED, fbGREEN, fbBLUE;
   
   fifo33 infifo
	 (.clkIn							(inClk),
	  .clkOut							(fbClk),
	  .reset							(inReset),
	  // Input port
	  .dataIn							({pgLineAddrIn[7:0], pgColAddrIn[8:0], pgRGB[15:0]}),
	  .dataValidIn						(pgValidPixelIn),
	  .takingDataIn						(pgTakingPixelIn),
	  // Output port
	  .dataOut							({fbLineAddrIn[7:0], fbColAddrIn[8:0], fbRGBIn[15:0]}),
	  .dataValidOut						(fbDataValidIn),
	  .takingDataOut					(fbTakingDataIn));

endmodule // inputAsyncInf
