/*************************************************************************/
/* File:        outputAsyncInf.v
 * Author:      Carl Ebeling
 *
 * Description: outputAsyncInf
 *              Provides a wrapper for the frame buffer -> display async fifo
 * Revsions:
 *   1.0        Initial design - 10/21/2009 - Carl Ebeling
 **************************************************************************/
`timescale 1ns/1ns

module outputAsyncInf
  (input fbClk,
   input outClk,   // Output data stream clock (display)
   input fbReset,
   input outReset,
   // Write (input) ports - Frame Buffer side
   input [23:0] fbRGBOut,
   input    fbDataValidOut,
   output   fbTakingDataOut,
   output   fbNewFrameOutInt,

   // Read (output) ports - Display Interface side
   output [23:0] dpRGB,
   input dpTakingPixelOut,
   input dpNewFrameOut
);
   
   parameter SIMULATE=0;

   reg             fbNewFrameOut;

   // Synchronizer - move newFrameOut from display to frame buffer
   reg   dpNewFrameOut1, dpNewFrameOutHold, fbNewFrameOutPrev;
   always @(posedge fbClk) begin
	  dpNewFrameOut1 <= dpNewFrameOutHold;
	  fbNewFrameOut <= dpNewFrameOut1;
	  fbNewFrameOutPrev <= fbNewFrameOut;
   end
   // Turn the newFrameOut signal into a one-clock signal
   assign fbNewFrameOutInt = fbNewFrameOut && !fbNewFrameOutPrev;

   reg fbNewFrameOutACK1, fbNewFrameOutACK, dpNewFrameOutACK;
   always @(posedge outClk) begin
	  fbNewFrameOutACK1 <= fbNewFrameOutACK;
	  dpNewFrameOutACK <= fbNewFrameOutACK1;
   end
	  
   // Implement the 4-phase handshake for newFrameOut
   always @(posedge outClk) begin
	  if (outReset) begin
		 dpNewFrameOutHold <= 0;
	  end else begin
		 if (dpNewFrameOut) begin
			dpNewFrameOutHold <= 1;
		 end
		 if (dpNewFrameOutACK) begin
			dpNewFrameOutHold <= 0;
		 end
	  end
   end // always @ (posedge outClk)
   // Implement the 4-phase handshake for newFrameOutACK
   always @(posedge fbClk) begin
	  if (fbReset) begin
		 fbNewFrameOutACK <= 0;
	  end else begin
		 fbNewFrameOutACK <= fbNewFrameOut;
	  end
   end

   fifo24 outfifo
	 (.clkIn							(fbClk),
	  .clkOut							(outClk),
	  .reset							(outReset | fbNewFrameOutInt),   // Reset FIFO on new frame
	  // Input port (frame buffer output)
	  .takingDataIn						(fbTakingDataOut),
	  .dataIn							(fbRGBOut[23:0]),
	  .dataValidIn						(fbDataValidOut),
	  // Output port (external output)
	  .dataOut							(dpRGB),
	  .dataValidOut						(  ),
	  .takingDataOut					(dpTakingPixelOut));

endmodule // outputAsyncInf
